#include "PWMChannel.hpp"

#include "driver/gpio.h"
#include "driver/ledc.h"



static bool timerConfigured = false;

static void configureTimer() {
  if (!timerConfigured) {
    ledc_timer_config_t ledc_timer;
    ledc_timer.duty_resolution = LEDC_TIMER_10_BIT, // resolution of PWM duty
    ledc_timer.freq_hz = 1000,                       // frequency of PWM signal; 2KHz is max for CN5711
    ledc_timer.speed_mode = LEDC_HIGH_SPEED_MODE,   // timer mode
    ledc_timer.timer_num = LEDC_TIMER_0,            // timer index
    // Required for newer esp-idf:
    //ledc_timer.clk_cfg = LEDC_AUTO_CLK,             // Auto select the source clock
    ledc_timer_config(&ledc_timer);

    ledc_fade_func_install(0);

    timerConfigured = true;
  }
}


CyanLight::PWMChannel::PWMChannel(uint8_t channel, uint8_t gpio) : channel{(ledc_channel_t)channel} {

  configureTimer();

  gpio_config_t conf;

  conf.intr_type = GPIO_INTR_DISABLE;
  conf.mode = GPIO_MODE_OUTPUT;
  conf.pin_bit_mask = 1ULL << gpio;
  conf.pull_up_en = GPIO_PULLUP_DISABLE;
  conf.pull_down_en = GPIO_PULLDOWN_DISABLE;

  gpio_config(&conf);
  gpio_set_level((gpio_num_t)gpio, 0);


  ledc_channel_config_t     ledc_channel;
  ledc_channel.channel    = this->channel;
  ledc_channel.duty       = 0;
  ledc_channel.gpio_num   = (gpio_num_t)gpio;
  ledc_channel.speed_mode = LEDC_HIGH_SPEED_MODE;
  ledc_channel.hpoint     = 0;
  ledc_channel.timer_sel  = LEDC_TIMER_0;

  ledc_channel_config(&ledc_channel);

  setBrightness(0);
}

void CyanLight::PWMChannel::setBrightness(float b) {

  uint16_t pwm = (uint16_t)( b*b * maxPWM );

  ledc_set_duty(LEDC_HIGH_SPEED_MODE, this->channel, pwm);
  ledc_update_duty(LEDC_HIGH_SPEED_MODE, this->channel);
}
